import * as pino from 'pino';
import { AnyIterable } from 'streaming-iterables';
import { Multiaddr } from '@multiformats/multiaddr';
import * as _libp2p_interface_peer_id from '@libp2p/interface-peer-id';
import { PeerId } from '@libp2p/interface-peer-id';
import * as _multiformats_multiaddr_dist_src from '@multiformats/multiaddr/dist/src';
import * as _libp2p_interface_connection_dist_src from '@libp2p/interface-connection/dist/src';
import { IncomingStreamData } from '@libp2p/interface-registrar';
import { Channel } from 'queueable';
import { Libp2p } from 'libp2p';

declare const newChannel: <T>(c: Channel<T>, i: IncomingStreamData, ctrl?: Channel<ControlMsg>) => {
    send: (value: T) => Promise<void>;
    done: (err?: Error) => Promise<void>;
    ctx: {
        id: StreamID;
        stat: {
            encryption: string;
            multiplexer: string;
            direction: _libp2p_interface_connection_dist_src.Direction;
            protocol: string;
            status: () => "OPEN" | "CLOSED";
        };
        remoteAddr: _multiformats_multiaddr_dist_src.Multiaddr;
        remotePeer: _libp2p_interface_peer_id.PeerId;
        tags: string[];
        metadata: Record<string, any>;
    };
};

type Chan<T> = ReturnType<typeof newChannel<T>>;
type IterableFunc<I, O> = (input: AnyIterable<I>, incomingData: IncomingStreamData) => AnyIterable<O> | Promise<AnyIterable<O>>;
type PeerAddr = string | Multiaddr | PeerId;
type Func<I, O> = (input: I, chan: Chan<O>) => Promise<void> | void;
type Service<I, O> = (chan: Chan<O>) => Promise<Func<I, O>> | Func<I, O> | void | Promise<void>;
interface Codec<T> {
    encoder: (data: T) => Uint8Array | Promise<Uint8Array>;
    decoder: (data: Uint8Array) => T | Promise<T>;
}
interface InitOptions<T> {
    codec?: Codec<T>;
}
interface TransportChannel<I, O> extends AsyncIterableIterator<I>, Chan<O> {
}
interface MetaPointError extends Error {
    type: "error";
    id: StreamID;
}
interface MetaPointSuccess {
    type: "success";
    id: StreamID;
}
type ControlMsg = MetaPointError | MetaPointSuccess;
interface StreamID {
    connection: string;
    stream: string;
}

declare const client: <T = any>(node: Libp2p, peer: PeerAddr | PeerAddr[], options?: InitOptions<T>) => Promise<(<I extends T, O extends T>(name: string) => Promise<((value: I) => Promise<O[]>) & TransportChannel<O, I>>)>;

declare const server: <T>(node: Libp2p, options?: InitOptions<T>) => Promise<{
    handle: <I extends T, O extends T>(name: string, func: Func<I, O>) => Promise<void>;
    serve: <I_1 extends T, O_1 extends T>(name: string, func: Service<I_1, O_1>) => Promise<void>;
}>;

declare const logger: pino.Logger<pino.LoggerOptions | pino.DestinationStream>;
declare const debug = false;

export { Chan, Codec, ControlMsg, Func, InitOptions, IterableFunc, MetaPointError, MetaPointSuccess, PeerAddr, Service, StreamID, TransportChannel, client, debug, logger, server };
